;; ws-background.jl - Change background accordingly to workspace
;; Based on rand-background.jl from David Tarkowski <dtarkows@rcn.com>
;; and ws-num.jl by Kevin D. Knerr  and others.
;; Author : Frdric Crozat <fcrozat@mandrakesoft.com>
;; Usage:
;;  Copy this file to somewhere in your sawfish load-path, for example
;;  "~/.sawfish/lisp".  If in doubt add (setq load-path (load-path
;;  "your-directory") with the parentheses and the quotes to your ~/.sawfishrc.
;;  Then add (require 'ws-background to your ~/.sawfishrc and restart
;;  sawfish or issue the (load "ws-background") from sawfish-client.
;;  The settings can be modified from the sawfish configuration panel.
;;  If you have not created a .sawfishrc before this, you should probably
;;  also add the line (require 'sawmill-defaults).
;;
;; License and Warranty;
;;  The code is released under the GNU General Public License v2 or later.
;;  You can find a copy of the GPL at <http://www.gnu.org/copyleft/gpl.html>
;;  This software is distributed in the hopes that it may be useful but without
;;  any warranty, without even the implied warranty of merchantability or
;;  fitness for a particular purpose.
;;
(require 'custom)
;;(require 'workspace)
(require 'sawfish.wm.workspace)

  (define (run-shell-command command)
    "Execute the given shell command."
    (system (format nil "%s &" command)))

  (define-command 'run-shell-command run-shell-command
    #:spec "sCommand:"
    #:type `(and (labelled ,(_ "Command:") string)))

;;This section is for the customization controls in sawmill-ui
(defgroup djt-ws-background "Workspace Background" :group workspace)

(defcustom djt-ws-background-enable nil 
   "Enable background image switching when changing workspaces?"
   :type boolean
   :user-level novice
   :group (workspace djt-ws-background)
)

(defcustom djt-ws-background-list '("~/.gnome/linux-mandrake-background1.png" 
				   "~/.gnome/linux-mandrake-background2.png" 
				   "~/.gnome/linux-mandrake-background3.png" 
				   "~/.gnome/linux-mandrake-background4.png" 
				   "~/.gnome/linux-mandrake-background5.png" 
				   "~/.gnome/linux-mandrake-background6.png" 
				   "~/.gnome/linux-mandrake-background7.png")
   "List of backgrounds for workspace."
   :type (list file)
   :depends djt-ws-background-enable
   :group (workspace djt-ws-background)
)

(defcustom djt-ws-background-set-command "/usr/bin/qiv -y"
   "The command used to set the background."
   :type string
   :depends djt-ws-background-enable
   :group (workspace djt-ws-background)
)

;;End of configuration section.

(defun djt-ws-background-hook ()
  (let ((card-background (length djt-ws-background-list)))
    (if (> card-background 0)
    (let* (
	   (space (mod (workspace-id-to-logical current-workspace) card-background))
	   (file (nth space djt-ws-background-list))
	   (background-command (concat djt-ws-background-set-command " " file )))
      (if (file-exists-p file)
	  (run-shell-command background-command)
	))
    )
  )
)

;;Adds or removes the hook to change the background as needed.
(define djt-ws-background-toggle 
  (lambda ()
    (if djt-ws-background-enable
	(and (add-hook 'enter-workspace-hook djt-ws-background-hook)
	     (add-hook 'after-initialization-hook djt-ws-background-hook))
      
      (and (remove-hook 'enter-workspace-hook djt-ws-background-hook)
	   (remove-hook 'after-initialization-hook djt-ws-background-hook))
   )
))

;;Very important!  Must initialize the hooks.
(djt-ws-background-toggle)

(custom-set-property 'djt-ws-background-enable ':after-set
		     djt-ws-background-toggle)

(provide 'ws-background)
